﻿using System;
using System.ComponentModel;
using System.Globalization;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Text.RegularExpressions;

namespace Sat2Ip.Library.Rtsp
{
    public class RtspClient : INotifyPropertyChanged
    {
        
        private string _rtspServerIp ;
        private string _rtspServerPort ;
        private Socket _rtspSocket;
        private int _rtspSequenceNum = 0;
        private string _rtspResponse;
        private string _rtspSessionId;
        private int _rtspSessionTTL;
        private string _rtspStreamId;
        private string _clientRtpPort ;
        private string _clientRtcpPort ;
        private string _serverRtpPort;
        private string _serverRtcpPort;
        private string _rtspStreamUrl;
        private string _destination;
        private string _source;
        private string _transport;

        public RtspClient()
        {
            
            _clientRtpPort = "52010";
            _clientRtcpPort = "52011";
        }

        public string Transport
        {
            get { return _transport; }
            set 
            {
                if (_transport != value)
                {
                    _transport = value;
                    OnPropertyChanged("Transport");
                }
            }
        }
        public string Destination
        {
            get { return _destination; }
            set
            {
                if (_destination != value)
                {
                    _destination = value;
                    OnPropertyChanged("Destination");
                }
            }
        }
        public string Source
        {
            get { return _source; }
            set
            {
                if (_source != value)
                {
                    _source = value;
                    OnPropertyChanged("Source");
                }
            }
        }
        public string ClientRtpPort
        {
            get { return _clientRtpPort; }
            set
            {
                if (_clientRtpPort != value)
                {
                    _clientRtpPort = value;
                    OnPropertyChanged("ClientRtpPort");
                }
            }
        }
        public string ClientRtcpPort
        {
            get { return _clientRtcpPort; }
            set
            {
                if (_clientRtcpPort != value)
                {
                    _clientRtcpPort = value;
                    OnPropertyChanged("ClientRtcpPort");
                }
            }
        }

        public string RtspServerIp
        {
            get { return _rtspServerIp; }
            set 
            {
                if (_rtspServerIp != value)
                {
                    if ((_rtspSocket!=null)&&(_rtspSocket.Connected))
                    {
                        Disconnect();
                    }
                    _rtspServerIp = value;
                    OnPropertyChanged("RtspServerIp");
                }
            }
        }
        public string RtspServerPort
        {
            get { return _rtspServerPort; }
            set
            {
                if (_rtspServerPort != value)
                {
                    if ((_rtspSocket != null) && (_rtspSocket.Connected))
                    {
                        Disconnect();
                    }
                    _rtspServerPort = value;
                    OnPropertyChanged("RtspServerPort");
                }
            }
        }

        private bool Connect()
        {
            _rtspSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            var ip = IPAddress.Parse(RtspServerIp);
            var port = Convert.ToInt16(RtspServerPort);
            var rtspEndpoint = new IPEndPoint(ip, port);
            _rtspSocket.Connect(rtspEndpoint);
            return _rtspSocket.Connected;
        }
        private bool Disconnect()
        {
            var result = false;
            if (_rtspSocket != null && _rtspSocket.Connected)
            {
                _rtspSocket.Shutdown(SocketShutdown.Both);
                _rtspSocket.Close();
                result = false;
            }
            return result;
        }

        public bool Setup(string query)
        {
            var result = false;
            if ((_rtspSocket==null))
            {
                Connect();
            }
            var sb = new StringBuilder();
            if (string.IsNullOrEmpty(_rtspSessionId))
            {
                sb.AppendFormat("{0} ", "SETUP");
                sb.AppendFormat("rtsp://{0}:{1}/?{2} RTSP/1.0\r\n", RtspServerIp, RtspServerPort, query);
                sb.AppendFormat("CSeq: {0}\r\n", (++_rtspSequenceNum).ToString(CultureInfo.InvariantCulture));
                sb.AppendFormat("Transport: RTP/AVP;{0};client_port={1}-{2}\r\n\r\n",_transport, ClientRtpPort, ClientRtcpPort);
            }
            else
            {
                sb.AppendFormat("{0} ", "SETUP");
                sb.AppendFormat("rtsp://{0}:{1}/?{2} RTSP/1.0\r\n", RtspServerIp, RtspServerPort, query);
                sb.AppendFormat("CSeq: {0}\r\n", (++_rtspSequenceNum).ToString(CultureInfo.InvariantCulture));
                sb.AppendFormat("Session: {0}\r\n", _rtspSessionId);
                sb.AppendFormat("Transport: RTP/AVP;{0};client_port={1}-{2}\r\n\r\n", _transport, ClientRtpPort, ClientRtcpPort);
            }
            result = SendRequest(sb.ToString());
            result = GetResponse();
            var lineArray = Regex.Split(_rtspResponse, @"[\r\n]+");
            foreach (var s in lineArray)
            {
                if(s.Contains("Date:"))
                {}
                if (s.Contains("Session:"))
                {
                    var sessionline = s;
                    if (string.IsNullOrEmpty(sessionline))
                    {

                    }
                    var sessionlinevalues = sessionline.Split(';');
                    var sessionvalues = sessionlinevalues[0].Split(':');
                    _rtspSessionId = sessionvalues[1];
                    if(sessionlinevalues.Length> 1)
                    {
                        var ttlvalues = sessionlinevalues[1].Split('=');
                        _rtspSessionTTL = int.Parse(ttlvalues[1]);
                    }
                    
                }
                if (s.Contains("Transport:"))
                {
                    var transportline = s;
                    if (string.IsNullOrEmpty(transportline))
                    {

                    }
                    var transportvalues = transportline.Split(';');
                    foreach (var transportvalue in transportvalues)
                    {
                        if (transportvalue.Contains("server_port"))
                        {
                            var serverportvalues = transportvalue.Substring(("server_port").Length)
                                                                 .Trim()
                                                                 .Split('=');

                            var ports = serverportvalues[1].Split('-');
                            _serverRtpPort = ports[0];
                            _serverRtcpPort = ports[1];
                        }
                        if (transportvalue.Contains("destination"))
                        {
                            var destinationvalues = transportvalue.Substring(("destination").Length)
                                                                 .Trim()
                                                                 .Split('=');
                            Destination = destinationvalues[1];
                        }
                        if (transportvalue.Contains("source"))
                        {

                        }
                    }
                }
                if (s.Contains("com.ses.streamID:"))
                {
                    _rtspStreamId = s.Substring(("com.ses.streamID:").Length).Trim();
                }
            }

            return result;
        }
        public bool Play(string query)
        {
            var result = false;
            if ((_rtspSocket == null) )
            {
                Connect();
            }
            var sb = new StringBuilder();
            if (string.IsNullOrEmpty(query))
            {
                sb.AppendFormat("{0} ", "PLAY");
                sb.AppendFormat("rtsp://{0}:{1}/stream={2} RTSP/1.0\r\n", RtspServerIp, RtspServerPort,
                                _rtspStreamId);
                sb.AppendFormat("CSeq: {0}\r\n", (++_rtspSequenceNum).ToString(CultureInfo.InvariantCulture));
                sb.AppendFormat("User-Agent: LibVLC/2.0.7 (Live555 Streaming Media v2012.12.18\r\n");
                sb.AppendFormat("Session: {0}\r\n\r\n", _rtspSessionId);

            }
            else
            {
                sb.AppendFormat("{0} ", "PLAY");
                sb.AppendFormat("rtsp://{0}:{1}/stream={2}?{3} RTSP/1.0\r\n", RtspServerIp, RtspServerPort,
                                _rtspStreamId, query);
                sb.AppendFormat("CSeq: {0}\r\n", (++_rtspSequenceNum).ToString(CultureInfo.InvariantCulture));
                sb.AppendFormat("User-Agent: LibVLC/2.0.7 (Live555 Streaming Media v2012.12.18\r\n");
                sb.AppendFormat("Session: {0}\r\n\r\n", _rtspSessionId);
            }
            result = SendRequest(sb.ToString());
            result = GetResponse();
            var lineArray = Regex.Split(_rtspResponse, @"[\r\n]+");
            foreach (var s in lineArray)
            {
                if (s.Contains("Session:"))
                {
                    var sessionline = s;
                    if (string.IsNullOrEmpty(sessionline))
                    {

                    }
                    var sessionvalues = sessionline.Split(':');
                    _rtspSessionId = sessionvalues[1];

                }
                if (s.Contains("RTP-Info:"))
                {
                    var rtpinfoline = s;
                    if (string.IsNullOrEmpty(rtpinfoline))
                    {

                    }

                    var rtpinfovalues = rtpinfoline.Split(' ');
                    _rtspStreamUrl = rtpinfovalues[1].Replace("url=", "");
                }
            }

            
            

            return result;
        }
        public bool Options()
        {
            var result = false;
            if ((_rtspSocket == null) )
            {
                Connect();
            }
            var sb = new StringBuilder();
            if (string.IsNullOrEmpty(_rtspSessionId))
            {
                sb.AppendFormat("{0} ", "OPTIONS");
                sb.AppendFormat("rtsp://{0}:{1}/ RTSP/1.0\r\n", RtspServerIp, RtspServerPort);
                sb.AppendFormat("CSeq: {0}\r\n\r\n", (++_rtspSequenceNum).ToString(CultureInfo.InvariantCulture));
            }
            else
            {
                sb.AppendFormat("{0} ", "OPTIONS");
                sb.AppendFormat("rtsp://{0}:{1}/ RTSP/1.0\r\n", RtspServerIp, RtspServerPort);
                sb.AppendFormat("CSeq: {0}\r\n", (++_rtspSequenceNum).ToString(CultureInfo.InvariantCulture));
                sb.AppendFormat("Session: {0}\r\n\r\n", _rtspSessionId);
            }
            result = SendRequest(sb.ToString());
            result = GetResponse();
            return result;
        }
        public bool Describe()
        {
            var result = false;
            if ((_rtspSocket == null) )
            {
                Connect();
            }
            var sb = new StringBuilder();
            if (string.IsNullOrEmpty(_rtspSessionId))
            {
                sb.AppendFormat("{0} ", "DESCRIBE");
                sb.AppendFormat("rtsp://{0}:{1}/ RTSP/1.0\r\n", RtspServerIp, RtspServerPort);
                sb.AppendFormat("CSeq: {0}\r\n", (++_rtspSequenceNum).ToString(CultureInfo.InvariantCulture));
                sb.AppendFormat("Accept: application/sdp\r\n\r\n");
            }
            else
            {
                sb.AppendFormat("{0} ", "DESCRIBE");
                sb.AppendFormat("rtsp://{0}:{1}/ RTSP/1.0\r\n", RtspServerIp, RtspServerPort);
                sb.AppendFormat("CSeq: {0}\r\n", (++_rtspSequenceNum).ToString(CultureInfo.InvariantCulture));
                sb.AppendFormat("Session: {0}\r\n", _rtspSessionId);
                sb.AppendFormat("Accept: application/sdp\r\n\r\n");
            }
            result = SendRequest(sb.ToString());
            result = GetResponse();
            var lineArray = Regex.Split(_rtspResponse, @"[\r\n]+");
            foreach (var s in lineArray)
            {
                if (s.Contains("v="))
                {

                }
                if (s.Contains("o="))
                {

                }
                if (s.Contains("s="))
                {

                }
                if (s.Contains("t="))
                {

                }
                if (s.Contains("m="))
                {

                }
                if (s.Contains("c="))
                {

                }
                if (s.Contains("a="))
                {

                }

            }

            return result;
        }
        public bool TearDown()
        {
            var result = false;
            if ((_rtspSocket == null) )
            {
                Connect();
            }
            var sb = new StringBuilder();
            sb.AppendFormat("{0} ", "TEARDOWN");
            sb.AppendFormat("rtsp://{0}:{1}/stream={2} RTSP/1.0\r\n", RtspServerIp, RtspServerPort, _rtspStreamId);
            sb.AppendFormat("CSeq: {0}\r\n", (++_rtspSequenceNum).ToString(CultureInfo.InvariantCulture));
            sb.AppendFormat("Session: {0}\r\n\r\n", _rtspSessionId);
            result = SendRequest(sb.ToString());
            Disconnect();
            return result;
        }

        private bool SendRequest(string request)
        {
            var result = false;
            if (_rtspSocket == null)
            {

            }
            try
            {
                var sendBuffer = Utils.StringToBytes(request);
                var sendBytesCount = _rtspSocket.Send(sendBuffer, sendBuffer.Length, SocketFlags.None);
                if (sendBytesCount < 1)
                {
                    result = false;
                }

                result = true;
            }
            catch (Exception e)
            {
                result = false;
            }
            return result;

        }
        private bool GetResponse()
        {
            var result = false;
            var revBytesCount = 0;
            var revBuffer = new byte[1024 * 4];
            _rtspResponse = string.Empty;
            _rtspSocket.ReceiveTimeout = 5000;
            while (true)
            {
                try
                {
                    revBytesCount = _rtspSocket.Receive(revBuffer, revBuffer.Length, SocketFlags.None);
                    if (revBytesCount >= 1)
                    {
                        break;
                    }
                    if (_rtspSocket.ReceiveTimeout >= 5000)
                    {
                        break;
                    }
                }
                catch (SocketException)
                {

                }
            }
            if (revBytesCount >= 1)
            {

                _rtspResponse = Utils.BytesToString(revBuffer, revBytesCount);
                if (_rtspResponse.StartsWith("RTSP/1.0"))
                {
                    var dstArray = _rtspResponse.Split(' ');
                    if (dstArray.Length > 1)
                    {
                        var code = dstArray[1];
                        if (code.Equals("200"))
                        {
                            result = true;
                        }
                    }
                }
            }

            return result;
        }

        #region Public Events

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region Protected Methods

        protected void OnPropertyChanged(string name)
        {
            var handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }

        #endregion

    }
}
