﻿using System;
using System.Windows.Forms;
using Sat2Ip.Library.Upnp;
using UPNPLib;

namespace Sat2Ip.UI.DiscoverySample
{
    public partial class Form1 : Form
    {

        private DeviceCollector _deviceCollector;
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            _deviceCollector = new DeviceCollector();
            _deviceCollector.DeviceAdded += DeviceAdded;
            _deviceCollector.SearchCompleted += SearchCompleted;
            toolStripProgressBar1.Style = ProgressBarStyle.Marquee;
        }

        public void DeviceAdded(UPnPDevice device)
        {

            var newnode = treeView1.Nodes[0].Nodes.Add(device.UniqueDeviceName, device.FriendlyName);
            newnode.ToolTipText = device.Description;
            if (device.HasChildren)
            {
                var childdevices = device.Children;
                foreach (UPnPDevice child in childdevices)
                {
                    newnode.Nodes.Add(child.UniqueDeviceName, child.FriendlyName).ToolTipText = child.Description;
                }
            }
            if (treeView1.Nodes[0].IsExpanded != true)
                treeView1.Nodes[0].Expand();
        }

        public void SearchCompleted()
        {
            toolStripProgressBar1.Style = ProgressBarStyle.Blocks;
        }

        public void DeviceRemoved(string sUDN)
        {
            treeView1.Nodes[sUDN].Remove();
        }

        private void treeDevices_AfterSelect(object sender, TreeViewEventArgs e)
        {
            toolStripProgressBar1.Style = ProgressBarStyle.Marquee;
            var finder = new UPnPDeviceFinderClass();
            var device = finder.FindByUDN(e.Node.Name);
            if (device != null)
            {
                textBox1.Text = device.FriendlyName;
                textBox2.Text = device.UniqueDeviceName;
                textBox4.Text = device.Description;
                textBox5.Text = device.ManufacturerName;
                textBox3.Text = device.Type;
                linkLabel1.Text = device.PresentationURL;
                if (linkLabel1.Links != null)
                {
                    linkLabel1.Links.Clear();
                }
                linkLabel1.Links.Add(new LinkLabel.Link(5,3,device.PresentationURL));
                if (linkLabel2.Links != null)
                {
                    linkLabel2.Links.Clear();
                }
                linkLabel2.Text = device.ManufacturerURL;
                linkLabel2.Links.Add(new LinkLabel.Link(6, 4, device.ManufacturerURL));
                string sImageUrl = device.IconURL("image/jpeg", 150, 150, 32);
                if (sImageUrl != null)
                {
                    pictureBox1.LoadAsync(sImageUrl);
                    pictureBox1.Visible = true;
                }
                else
                    pictureBox1.Visible = false;
                listView1.Items.Clear();
                var services = device.Services;
                if (services != null)
                {
                    foreach (UPnPService service in services)
                    {
                        var item = new ListViewItem {Text = service.Id, Tag = service.Id};
                        item.SubItems.Add(service.ServiceTypeIdentifier);
                        listView1.Items.Add(item);
                    }
                }
                webBrowser1.Url = new Uri(((IUPnPDeviceDocumentAccess)device).GetDocumentURL());
                tabControl1.Visible = true;
            }
            toolStripProgressBar1.Style = ProgressBarStyle.Blocks;
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }
        private void linkLabel_LinkClicked(object sender, System.Windows.Forms.LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start(e.Link.LinkData.ToString());
        }

    }
}

