﻿using System;
using System.Windows.Forms;
using Sat2Ip.Library.Rtsp;


namespace Sat2Ip.UI.RtspSample
{
    public partial class Form1 : Form
    {
        private readonly RtspClient _client;
        private Boolean _isstreaming = false;
        private readonly Timer _keepaLiveTimer;
        public Form1()
        {
            InitializeComponent();
            // Initialize a new Instanze of RtspClient
            _client = new RtspClient();
            //Setup KeepAliveTimer
            _keepaLiveTimer = new Timer { Interval = 60000, Enabled = true };
            _keepaLiveTimer.Tick += _keepaLiveTimer_Tick;
        }

        void _keepaLiveTimer_Tick(object sender, EventArgs e)
        {
            _client.Options();
        }
        private void Form1_Load(object sender, EventArgs e)
        {
            //Fill the ServerAddress TextBox with Data
            tbRtspAddress.Text = "192.168.2.109";
            tbRtspPort.Text = "554";
            //Fill the Service ComboBox with Data
            cbService.Items.Add(new Service("3Sat HD", "1","1", "11347", "22000", "v", "dvbs2", string.Join(",", new[] { "0", "6500", "6510", "6520", "6521", "6523", "6522" })));
            cbService.Items.Add(new Service("ZDF HD", "1", "1", "11361", "22000", "h", "dvbs2", string.Join(",", new[] { "0", "6100", "6110", "6120", "6121", "6123", "6122" })));
            cbService.Items.Add(new Service("ZDF neo HD", "1", "1", "11361", "22000", "h", "dvbs2", string.Join(",", new[] { "0", "6300", "6310", "6320", "6321", "6323", "6322" })));
            cbService.Items.Add(new Service("Das Erste HD", "1", "1", "11493", "22000", "h", "dvbs2", string.Join(",", new[] { "0", "5100", "5101", "5102", "5103", "5106" })));
            cbService.SelectedIndex = 0;
            // Fill the TransportMode ComboBox with Data
            cbTransportMode.Items.Add("unicast");
            cbTransportMode.Items.Add("multicast");
            cbTransportMode.SelectedIndex = 0;
            //Fill the Rtp and Rtcp TextBoxes
            tbRtpPort.Text = _client.ClientRtpPort;
            tbRtcpPort.Text = _client.ClientRtcpPort;

        }

        private void cbService_SelectedIndexChanged(object sender, EventArgs e)
        {
            var service = cbService.SelectedItem;
            if (!_isstreaming)
            {
                _keepaLiveTimer.Stop();
                _client.Setup(service.ToString());
                _client.Play(String.Empty);
                _isstreaming = true;
                _keepaLiveTimer.Start();
            }
            else
            {
                _keepaLiveTimer.Stop();
                _client.Play(service.ToString());
                _keepaLiveTimer.Start();
            }
            
            axWindowsMediaPlayer1.URL = string.Format("rtp://{0}:{1}", _client.Destination, _client.ClientRtpPort);
            
        }

        private void cbTransportMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            _client.Transport = cbTransportMode.SelectedItem.ToString();
        }

        private void tbRtspAddress_TextChanged(object sender, EventArgs e)
        {
            _client.RtspServerIp = tbRtspAddress.Text;
        }

        private void tbRtspPort_TextChanged(object sender, EventArgs e)
        {
            _client.RtspServerPort = tbRtspPort.Text;
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            _client.TearDown();
        }
    }
}
